----------------------------------------------------------------------------------
-- Company: fumi2kick
-- Engineer: rerofumi
-- 
-- Create Date:    01:37:05 06/14/2007 
-- Design Name: 
-- Module Name:    video_crtc - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity video_crtc is
    Port ( BA : out  STD_LOGIC_VECTOR (16 downto 0);
           BD : in  STD_LOGIC_VECTOR (7 downto 0);
           VBLNK : out  STD_LOGIC;
           VBLNK_IN : out  STD_LOGIC;
           CLK : in  STD_LOGIC;
           VOUT1 : out  STD_LOGIC;
           VOUT2 : out  STD_LOGIC);
end video_crtc;

architecture Behavioral of video_crtc is
  -- wire
  signal HSYNC_SIG : STD_LOGIC;
  signal HBLANK_SIG : STD_LOGIC;
  signal VSYNC_SIG : STD_LOGIC;
  signal VBLANK_SIG : STD_LOGIC;
  signal DOT_CLK : STD_LOGIC;
  signal DOT_ADDRESS : STD_LOGIC_VECTOR (16 downto 0);
  signal HCOUNTER : integer range 0 to 910;
  signal VCOUNTER : integer range 0 to 272;

begin

  -------------------------------------
  -- H Counter
  process(CLK)
  begin
    if (CLK'event and CLK='1') then
      -- counter
      HCOUNTER <= HCOUNTER + 1;
      if (HCOUNTER = 910) then
        HCOUNTER <= 0;
      end if;
      -- H Sync (from 156+57 to 910-57 dotarea 640)
      if (HCOUNTER<(156+57+1) or HCOUNTER>(910-57)) then
        HBLANK_SIG <= '1';
        if (HCOUNTER>21 and HCOUNTER<89) then
          HSYNC_SIG <= '1';
        else
          HSYNC_SIG <= '0';
        end if;
      else
        -- dot area (CLK/2)
        HBLANK_SIG <= '0';
        if DOT_CLK='1' then
          DOT_CLK <= '0';
        else
          DOT_CLK <= '1';
        end if;         
      end if;
    end if;
  end process;

  -------------------------------------
  -- V Counter
  process(HBLANK_SIG)
  begin
    if (HBLANK_SIG'event and HBLANK_SIG='1') then
      -- counter
      VCOUNTER <= VCOUNTER + 1;
      if (VCOUNTER = 261) then
        VCOUNTER <= 0;
      end if;
      -- V Blank timing
      if VCOUNTER<21 then
        VBLNK <= '1';
      else
        VBLNK <= '0';
      end if;
      -- V Sync
      if VCOUNTER<22 then
        VBLANK_SIG <= '1';
        if VCOUNTER>3 and VCOUNTER<7 then
          VSYNC_SIG <= '1';
        else
          VSYNC_SIG <= '0';
        end if;
      else
        VBLANK_SIG <= '0';
      end if;
    end if;
    VBLNK_IN <= VBLANK_SIG;
  end process;


  -------------------------------------
  -- DATA ADDRESS Counter
  process(VBLANK_SIG, DOT_CLK)
  begin
    if VBLANK_SIG='1' then
      DOT_ADDRESS <= "00000000000000000";
    else
      if (DOT_CLK'event and DOT_CLK='1') then
        DOT_ADDRESS <= DOT_ADDRESS + "1";
      end if;
    end if;
    BA <= DOT_ADDRESS;
  end process;


  -------------------------------------
  -- MIXER and output
  process(HSYNC_SIG, HBLANK_SIG, VSYNC_SIG, VBLANK_SIG)
  begin
    if (HSYNC_SIG='1' or VSYNC_SIG='1') then
      -- sync
      VOUT1 <= '0';
      VOUT2 <= '0';
    else
      if (HBLANK_SIG='1' or VBLANK_SIG='1') then
        -- black
        VOUT1 <= '0';
        VOUT2 <= '1';
      else
        -- pixel output
        if BD(7)='1' then
          if BD="11111111" then
            VOUT1 <= '1';
            VOUT2 <= '1';
          else
            VOUT1 <= '1';
            VOUT2 <= '0';
          end if;
        else
          VOUT1 <= '0';
          VOUT2 <= '1';
        end if;
      end if;
    end if;
  end process;

end Behavioral;

